<?php

namespace EventPortal\Tests\Unit\Util;

use Carbon\Carbon;
use EventPortal\Domain\ActionList\Models\ActionList;
use EventPortal\Domain\ActionList\Models\Task;
use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;

class TaskTest extends TestCase
{
    use DatabaseMigrations;

    /** @test */
    public function can_get_completed_bool()
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id
        ]);
        $task = factory(Task::class)
            ->state('completed')
            ->create([
                'action_list_id' => $list->id
            ]);

        $this->assertTrue($task->isComplete);
        $task->update(['completed_at' => null]);
        $this->assertFalse($task->isComplete);
    }

    /** @test */
    public function can_complete()
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id
        ]);
        $admin = factory(Admin::class)->create();
        $task = factory(Task::class)
            ->create([
                'action_list_id' => $list->id
            ]);

        $this->assertTrue($task->complete($admin->user));
        $this->assertTrue($task->isComplete);
        $this->assertSameModel($admin->user, $task->completedBy);
        $this->assertNotNull($task->completed_at);
    }

    /** @test */
    public function can_uncomplete()
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id
        ]);
        $task = factory(Task::class)
            ->state('completed')
            ->create([
                'action_list_id' => $list->id
            ]);

        $this->assertFalse($task->uncomplete());
        $this->assertFalse($task->isComplete);
        $this->assertNull($task->completed_at);
        $this->assertNull($task->completedBy);
    }
    /** @test */
    public function can_toggle_status()
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id
        ]);
        $admin = factory(Admin::class)->create();
        $task = factory(Task::class)->create(['action_list_id' => $list->id]);

        $this->assertFalse($task->isComplete);

        $this->assertTrue($task->toggle($admin->user));
        $task->fresh();

        $this->assertTrue($task->isComplete);
        $this->assertEquals(get_class($task->completed_at), Carbon::class);

        $this->assertFalse($task->toggle($admin->user));
        $task->fresh();

        $this->assertFalse($task->isComplete);
        $this->assertNull($task->completed_at);
    }

    /** @test */
    public function can_get_completed_by_user()
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id
        ]);
        $admin = factory(Admin::class)->create();
        $task = factory(Task::class)->create(['action_list_id' => $list->id]);

        $this->assertNull($task->completedBy);
        $task->toggle($admin->user);
        $task->fresh();

        $this->assertSameModel($admin->user, $task->completedBy);
        $this->assertEquals(get_class($task->completed_at), Carbon::class);

        $task->toggle($admin->user);
        $task->fresh();

        $this->assertNull($task->completedBy);
    }

    /** @test */
    public function can_get_parent_list()
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id
        ]);
        $task = factory(Task::class)->create(['action_list_id' => $list->id]);

        $this->assertSameModel($list, $task->actionList);
    }

    /** @test */
    public function complete_scope_works()
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id
        ]);
        $list->tasks()
            ->saveMany(factory(Task::class, 5)
                ->state('completed')
                ->make());

        $list->tasks()
            ->saveMany(factory(Task::class, 4)
                ->make());

        $this->assertCount(9, $list->tasks);
        $this->assertCount(5, $list->tasks()->completed()->get());
    }

    /** @test */
    public function incompleted_scope_works()
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id
        ]);
        $list->tasks()
            ->saveMany(factory(Task::class, 5)
                ->state('completed')
                ->make());

        $list->tasks()
            ->saveMany(factory(Task::class, 4)
                ->make());

        $this->assertCount(9, $list->tasks);
        $this->assertCount(4, $list->tasks()->incompleted()->get());
    }
}
